/* copyright © by Stefan Voitel, 2003 - all rights reserved                   */
/* eMail : stefan.voitel@winways.de                                           */

#include "AgtSvr.h"
#include "cAgentCommands.h"
#include "cMMControl.h"
#include "defs.h"

using namespace std;

char* toLower(char* ch) {
    return const_cast<char*>(jeliza::string(ch).get_lower().ref().c_str());
}

char*& ConvertCharLower(char*& ch) {
    char* ch2 = const_cast<char*>(jeliza::string(ch).get_lower().ref().c_str());
    ch = ch2;
    return ch;
}

CHAR narrow_to_wide(char w) {
    // simple typecast
    // works because UNICODE incorporates ASCII into itself
    return CHAR(w);
}
CHAR* toWChar(string str) {
    string wstr;
    std::transform(str.begin(), str.end(), wstr.begin(), narrow_to_wide);
    return const_cast<CHAR*>(wstr.c_str());
}

WCHAR narrow_to_wideR(char w) {
    // simple typecast
    // works because UNICODE incorporates ASCII into itself
    return WCHAR(w);
}
WCHAR* toWCharR(string str) {
    wstring wstr;
    std::transform(str.begin(), str.end(), wstr.begin(), narrow_to_wideR);
    return const_cast<WCHAR*>(wstr.c_str());
}

cAgentCommands::cAgentCommands (void) :
                iCharacterEx(NULL),CharacterID(-1),StopPlaying(false) {

if ((VolumeCtrlMaster = new cMMControl()) != NULL) {
   if (! VolumeCtrlMaster->Open (cMMControl::srcNone,cMMControl::dstSpeakers,
                                    cMMControl::fdVolume,cMMControl::swMute)) {
      delete VolumeCtrlMaster;
      VolumeCtrlMaster = NULL;
      }
   }

if ((VolumeCtrlWave = new cMMControl()) != NULL) {
   if (! VolumeCtrlWave->Open (cMMControl::srcWaveout,cMMControl::dstSpeakers,
                                    cMMControl::fdVolume,cMMControl::swMute)) {
      delete VolumeCtrlWave;
      VolumeCtrlWave = NULL;
      }
   }
}

cAgentCommands::~cAgentCommands (void) {
if (! IsBadCodePtr ((FARPROC) VolumeCtrlMaster))
   delete VolumeCtrlMaster;

if (! IsBadCodePtr ((FARPROC) VolumeCtrlWave))
   delete VolumeCtrlWave;
} // --------------------------------------------------------------------------



bool cAgentCommands::Speak      (char *Text) {
bool trigger = true;
if (iCharacterEx != NULL) {
   int      len   = (int) strlen (Text);
   wchar_t *speak = new wchar_t[len +1];

   if (MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,Text,-1,speak,
                                                   sizeof (wchar_t) *len)) {
      BSTR  p = SysAllocString (speak);
      trigger = FAILED (iCharacterEx->Speak (p,NULL,&RequestID));
      SysFreeString (p);
      }
   delete[] speak;
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Think      (char *Text) {
bool trigger = true;
if (iCharacterEx != NULL) {
   int      len   = (int) strlen (Text);
   wchar_t *think = new wchar_t[len +1];

   if (MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,Text,-1,think,
                                                   sizeof (wchar_t) *len)) {
      BSTR p = SysAllocString (think);
      trigger = FAILED (iCharacterEx->Think (p,&RequestID));
      SysFreeString (p);
      }
   delete think;
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Play       (char *Animation) {
bool trigger = true;
if (iCharacterEx != NULL) {
   int      len   = (int) strlen (Animation);
   wchar_t *play  = new wchar_t[len +1];

   if (MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,Animation,-1,play,
                                                   sizeof (wchar_t) *len)) {
      BSTR  p = SysAllocString (play);
      trigger = FAILED (iCharacterEx->Play (p,&RequestID));
      SysFreeString (p);
      }
   delete[] play;
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Name       (char *Name) {
if (iCharacterEx != NULL) {
   int      len   = (int) strlen (Name);
   wchar_t *name  = new wchar_t[len +1];

   if (MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,Name,-1,name,
                                                   sizeof (wchar_t) *len)) {
      BSTR p = SysAllocString (name);
      iCharacterEx->SetName (p);
      SysFreeString (p);
      }
   delete[] name;
   }
return true;
}              // ---------------------

bool cAgentCommands::Move       (char *DeltaPosition) {
bool trigger = true;
if (iCharacterEx != NULL) {
   long  curx,cury;

   if (SUCCEEDED(iCharacterEx->GetPosition (&curx,&cury))) {
      int   x,y,s;
      char *p;
      if (*(p= nextalnum (DeltaPosition))) {
         x = atoi (p) +curx;
         if (*(p = nextwhite(nextalnum (p)))) {
            y = atoi (p) +cury;
            s = atoi (nextwhite(nextalnum (p)));
            trigger = FAILED(iCharacterEx->MoveTo ((short) x,(short) y,s,
                                                                  &RequestID));
            }
         }
      }
   }
return trigger;
}              // ---------------------

bool cAgentCommands::MoveTo     (char *Position) {
bool trigger = true;
if (iCharacterEx != NULL) {
   int   x = 0,y = 0,s = 0;
   long  h,w;
   bool  success = true;

   Position = toLower(Position);
   iCharacterEx->GetSize (&w,&h);

   if (nextarg (Position,"righttop")) {
      x = GetSystemMetrics (SM_CXFULLSCREEN) -w;
      }
   else if (nextarg (Position,"leftbottom")) {
      y = GetSystemMetrics (SM_CYFULLSCREEN) -h;
      }
   else if (nextarg (Position,"rightbottom")) {
      x = GetSystemMetrics (SM_CXFULLSCREEN) -w;
      y = GetSystemMetrics (SM_CYFULLSCREEN) -h;
      }
   else if (nextarg (Position,"center")) {
      x = (GetSystemMetrics (SM_CXFULLSCREEN) -w) /2;
      y = (GetSystemMetrics (SM_CYFULLSCREEN) -h) /2;
      }
   else if (! nextarg (Position,"lefttop")) {
      success = false;
      }

   if (success) {
      s = atoi  (nextwhite (nextalnum (Position)));
      }

   if (! success) {
      char *p;
      if (*(p= nextalnum(Position))) {
         x = atoi  (p);
         if (*(p = nextwhite(nextalnum (p)))) {
            y = atoi  (p);
            s = atoi (nextwhite(nextalnum (p)));
            success = true;
            }
         }
      }

   if (success) {
      trigger = FAILED(iCharacterEx->MoveTo((short) x,(short) y,s,&RequestID));
      }
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Show       (char *Speed) {
bool trigger = true;
if (iCharacterEx != NULL) {
   ConvertCharLower (Speed);
   trigger = FAILED(iCharacterEx->Show(nextarg(Speed,"fast"),&RequestID));
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Hide       (char *Speed) {
bool trigger = true;
if (iCharacterEx != NULL) {
   ConvertCharLower (Speed);
   trigger = FAILED(iCharacterEx->Hide(nextarg(Speed,"fast"),&RequestID));
   }
return trigger;
}              // ---------------------

bool cAgentCommands::Size       (char *NewSize) {
if (iCharacterEx != NULL) {
   char *p;
   int x = atoi  (p = nextalnum (NewSize));
   int y = atoi  (nextwhite (nextalnum (p)));
   iCharacterEx->SetSize (x,y);
   }
return true;
}              // ---------------------
#pragma warning (push)
#pragma warning (disable : 4100)

#if defined (__BORLANDC__)
#pragma argsused
#endif

bool cAgentCommands::Resize     (char *DummyArg) {
if (iCharacterEx != NULL) {
   long x,y;
   iCharacterEx->GetOriginalSize (&x,&y);
   iCharacterEx->SetSize (x,y);
   }
return true;
}              // ---------------------

#if defined (__BORLANDC__)
#pragma argsused
#endif

bool cAgentCommands::Stop       (char *DummyArg) {

StopPlaying = true;
return true;
}              // ---------------------

#pragma warning (pop)

bool cAgentCommands::Language   (char *Language) {
ConvertCharLower (Language);
if (iCharacterEx != NULL) {
   if      (nextarg (Language,"german"))
      iCharacterEx->SetLanguageID(MAKELANGID(LANG_GERMAN,SUBLANG_GERMAN));
   else if (nextarg (Language,"american"))
      iCharacterEx->SetLanguageID(MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US));
   else if (nextarg (Language,"english"))
      iCharacterEx->SetLanguageID(MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_UK));
   else
      iCharacterEx->SetLanguageID(MAKELANGID(LANG_NEUTRAL,SUBLANG_SYS_DEFAULT));
   }
return true;
}              // ---------------------

bool cAgentCommands::Menu       (char *Enable) {
if (iCharacterEx != NULL) {
   ConvertCharLower (Enable);
   iCharacterEx->SetAutoPopupMenu (nextarg (Enable,"on"));
   }
return true;
}              // ---------------------

bool cAgentCommands::ShellRun   (char *CmdLine) {
UINT  Result;
char  Command [MAX_PATH *2 +4];

ZeroMemory (Command,sizeof(Command));
strcpy (Command,CmdLine);

char *verb;
char *p = Command;

if (*Command == '\"') {
   verb = &Command[1];
   p = verb;
   while (*p && *p != '\"') p++;

   if (p)
      *p ++ = '\0';
   }
else verb = NULL;

char *file = nextalnum (p);

if ((Result = (UINT)(LPARAM) ShellExecute(NULL, toWChar(verb),toWChar(file),0,0,SW_SHOW)) <= 32) {
   char e[255], m[1024];
   FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM,NULL,Result,0,toWChar(e),sizeof(toWChar(e)),NULL);
//   wsprintf (m,"%s\r\n%s",CmdLine,e);
//   MessageBox (NULL,m,"command fails",MB_ICONINFORMATION);
    cout << "command fails" << endl;
   }
return true;
}              // ---------------------

bool cAgentCommands::Execute    (char *CmdLine) {
STARTUPINFO          si;
PROCESS_INFORMATION  pi;

ZeroMemory (&si,sizeof(si));
ZeroMemory (&pi,sizeof(pi));

si.cb          = sizeof(si);
si.dwFlags     = STARTF_USESHOWWINDOW;
si.wShowWindow = SW_SHOW;

if (CreateProcess (NULL,toWChar(CmdLine),NULL,NULL,false,STARTF_FORCEONFEEDBACK,
                                                      NULL,NULL,&si,&pi)) {
   CloseHandle (pi.hThread);
   WaitForSingleObject (pi.hProcess,INFINITE);
   }
else {
   char e[255], m[1024];  DWORD Result = GetLastError();
//   FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM,NULL,Result,0,e,sizeof(e),NULL);
//   wsprintf (m,"%s\r\n%s",CmdLine,e);
//   MessageBox (NULL,m,"command fails",MB_ICONINFORMATION);
    cout << "command fails" << endl;
   }
return true;
}              // ---------------------

bool cAgentCommands::Sounds     (char *Switch) {
if (iCharacterEx != NULL) {
   ConvertCharLower (Switch);
   iCharacterEx->SetSoundEffectsOn (! nextarg(Switch,"off"));
   }
return true;
}              // ---------------------

bool cAgentCommands::Volume     (char *Channel) {
cMMControl *mmc = NULL;
ConvertCharLower (Channel);

if (nextarg(Channel,"master"))
   mmc = VolumeCtrlMaster;
else if (nextarg(Channel,"wave"))
   mmc = VolumeCtrlWave;

if (mmc != NULL) {
   char *args = nextalnum(nextwhite(Channel));

   if (! *args || nextarg (args,"on") || nextarg (args,"off")) {
      mmc->Switch (! nextarg (args,"off"));
      }
   else if (nextarg (args,"reset")) {
      mmc->Reset();
      }
   else {
      mmc->Fade (atoi(args));
      }
   }
return true;
}              // ---------------------

// EOF
